unit EingabeForm;


{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  StdCtrls, Buttons, Types;

type

  { TEingabeEinheit }

  TEingabeEinheit = class(TPanel)
  private
    BtnUp, BtnDown: TBitBtn;
    FWert: char;
    procedure EingabeEinheitMouseWheel(Sender: TObject; Shift: TShiftState;
      WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
    procedure SetWert(AValue: char);
    procedure BtnClick(Sender: TObject);
    procedure EinageEinheitPaint(Sender: TObject);
    procedure EinageEinheitResize(Sender: TObject);
    procedure Up;procedure Down;
  public
    property Wert: char read FWert write SetWert;
    constructor Create(TheOwner: TComponent); override;
  end;


  { TEingabeForm }

  TEingabeForm = class(TForm)
  private
  var
    anzEinheiten: integer;
    OkBtn, CancelBtn: TBitBtn;
    EingabeEinheit: array of TEingabeEinheit;

    function GetWert: string;
    procedure SetWert(AValue: string);
  public
    property Wert: string read GetWert write SetWert;
    constructor CreateNew(TheOwner: TComponent; AanzEinheiten: integer = 4);
  end;


implementation

{ TEingabeEinheit }

procedure TEingabeEinheit.EinageEinheitResize(Sender: TObject);
var
  size: integer;
begin
  size := Width - 6;

  with BtnUp do begin
    Left := 3;
    Top := 3;
    Width := size;
    Height := size;
    Parent := Self;
  end;

  with BtnDown do begin
    Left := 3;
    Top := Self.Height - size - 3;
    Width := size;
    Height := size;
    Parent := Self;
  end;
end;

procedure TEingabeEinheit.Up;
begin
  Inc(FWert);
  if FWert > '9' then begin
    FWert := '0';
  end;
end;

procedure TEingabeEinheit.Down;
begin
  Dec(FWert);
  if FWert < '0' then begin
    FWert := '9';
  end;
end;

procedure TEingabeEinheit.BtnClick(Sender: TObject);
begin
  if Sender = BtnUp then begin
    Up;
  end;
  if Sender = BtnDown then begin
    Down;
  end;
  Invalidate;
end;

procedure TEingabeEinheit.EinageEinheitPaint(Sender: TObject);
var
  s: TSize;
begin
  Canvas.Brush.Style := bsClear;
  Canvas.Font.Height := 76;
  s := Canvas.TextExtent(Wert);
  Canvas.TextOut((Width - s.cx) div 2, (Height - s.cy) div 2, Wert);
end;

procedure TEingabeEinheit.SetWert(AValue: char);
begin
  FWert := AValue;
  Invalidate;
end;

procedure TEingabeEinheit.EingabeEinheitMouseWheel(Sender: TObject;
  Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
begin
  if WheelDelta < 0 then begin
    Up;
  end;
  if WheelDelta > 0 then begin
    Down;
  end;
  Invalidate;
end;

constructor TEingabeEinheit.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  BtnUp := TBitBtn.Create(Self);
  BtnUp.Glyph.LoadFromFile('up.bmp');
  BtnUp.OnClick := @BtnClick;

  BtnDown := TBitBtn.Create(Self);
  BtnDown.Glyph.LoadFromFile('down.bmp');
  BtnDown.OnClick := @BtnClick;

  OnResize := @EinageEinheitResize;
  OnPaint := @EinageEinheitPaint;
  OnMouseWheel:=@EingabeEinheitMouseWheel;

end;

{ TEingabeForm }

function TEingabeForm.GetWert: string;
var
  i: integer;
begin
  SetLength(Result, anzEinheiten);
  for i := 0 to anzEinheiten - 1 do begin
    Result[i + 1] := EingabeEinheit[i].Wert;
  end;
end;

procedure TEingabeForm.SetWert(AValue: string);
var
  l, l2, i: integer;
begin
  if Length(AValue) < anzEinheiten then begin
    l := Length(AValue);
  end else begin
    l := anzEinheiten;
  end;
  l2 := anzEinheiten - Length(AValue);
  if l2 < 0 then begin
    l2 := 0;
  end;
  for i := 0 to l - 1 do begin
    EingabeEinheit[i + l2].Wert := AValue[i + 1];
  end;
end;

constructor TEingabeForm.CreateNew(TheOwner: TComponent; AanzEinheiten: integer);
const
  w = 50;
var
  i: integer;
begin
  inherited CreateNew(TheOwner);
  anzEinheiten := AanzEinheiten;
  Position := poScreenCenter;
  Height := 250;
  Width := w * anzEinheiten + 10;
  BorderStyle := bsDialog;
  Constraints.MinWidth := 95 + 80 + 10;

  SetLength(EingabeEinheit, anzEinheiten);
  for i := 0 to anzEinheiten - 1 do begin
    EingabeEinheit[i] := TEingabeEinheit.Create(Self);
    with EingabeEinheit[i] do begin
      Top := 5;
      Left := w * i + 5;
      Width := w;
      Height := Self.Height - 55;
      Parent := Self;
      Wert := '0';
    end;
  end;

  OkBtn := TBitBtn.Create(Self);
  with OkBtn do begin
    Parent := Self;
    Caption := 'Ok';
    Kind := bkOK;
  end;

  CancelBtn := TBitBtn.Create(Self);
  with CancelBtn do begin
    Parent := Self;
    Caption := 'Cancel';
    Kind := bkCancel;
  end;

  with OkBtn do begin
    Left := Self.Width - 85;
    Top := Self.Height - 40;
    Width := 80;
    Height := 30;
  end;

  with CancelBtn do begin
    Left := Self.Width - 85 - 90;
    Top := Self.Height - 40;
    Width := 80;
    Height := 30;
  end;
end;

end.
